@@ -2,37 +2,42 @@
 # -*- coding: utf-8 -*-
 # vim: ai ts=4 sts=4 et sw=4 nu
 
-import dateutil.parser
+import concurrent.futures
+import datetime
 import json
-import time
+import locale
 import pathlib
 import shutil
 import tempfile
-import datetime
+import time
 import urllib.parse
 
+import dateutil.parser
 import jinja2
 from bs4 import BeautifulSoup
-from zimscraperlib.zim import make_zim_file
-from zimscraperlib.i18n import get_language_details
-from zimscraperlib.download import save_large_file
-from zimscraperlib.video.presets import VideoWebmLow, VideoMp4Low
+from slugify import slugify
 from kiwixstorage import KiwixStorage
 from pif import get_public_ip
+from zimscraperlib.download import BestMp4, BestWebm, YoutubeDownloader, save_large_file
+from zimscraperlib.i18n import get_language_details, setlocale, _
+from zimscraperlib.image.optimization import optimize_image
+from zimscraperlib.image.presets import WebpMedium
+from zimscraperlib.image.transformation import resize_image
+from zimscraperlib.video.presets import VideoMp4Low, VideoWebmLow
+from zimscraperlib.zim import make_zim_file
 
-from .utils import download_link, update_subtitles_list, WebVTT
 from .constants import (
+    ALL,
+    BASE_URL,
+    MATCHING,
+    NONE,
     ROOT_DIR,
     SCRAPER,
-    BASE_URL,
-    NONE,
-    MATCHING,
     TEDLANGS,
-    ALL,
     getLogger,
 )
 from .processing import post_process_video
-
+from .utils import WebVTT, download_link, update_subtitles_list
 
 logger = getLogger()
 
@@ -49,6 +54,7 @@ class Ted2Zim:
         no_zim,
         fname,
         languages,
+        locale_name,
         title,
         description,
         creator,
@@ -62,6 +68,7 @@ class Ted2Zim:
         subtitles_enough,
         subtitles_setting,
         tmp_dir,
+        threads,
     ):
 
         # video-encoding info
@@ -104,6 +111,8 @@ class Ted2Zim:
                 [lang.strip() for lang in subtitles_setting.split(",")]
             )
         )
+        self.threads = threads
+        self.yt_downloader = None
 
         # optimization cache
         self.s3_url_with_credentials = s3_url_with_credentials
@@ -126,6 +135,18 @@ class Ted2Zim:
         self.zim_lang = None
         self.already_visited = []
 
+        # set and record locale for translations
+        locale_details = get_language_details(locale_name)
+        if locale_details["querytype"] != "locale":
+            locale_name = locale_details["iso-639-1"]
+        try:
+            self.locale = setlocale(ROOT_DIR, locale_name)
+        except locale.Error:
+            logger.error(
+                f"No locale for {locale_name}. Use --locale to specify it. defaulting to en_US"
+            )
+            self.locale = setlocale(ROOT_DIR, "en")
+
     @property
     def templates_dir(self):
         return ROOT_DIR.joinpath("templates")
@@ -151,10 +172,10 @@ class Ted2Zim:
         return BASE_URL + "playlists"
 
     def append_part1_or_part3(self, lang_code_list, lang_info):
-        """ Fills missing ISO languages codes for all in list
+        """Fills missing ISO languages codes for all in list
 
-            lang_code_list: list og lang codes
-            lang_info: see zimscraperlib.i18n """
+        lang_code_list: list og lang codes
+        lang_info: see zimscraperlib.i18n"""
 
         # ignore extra language mappings if supplied query was an iso-639-1 code
         if "part1" in lang_info["iso_types"]:
@@ -175,11 +196,11 @@ class Ted2Zim:
                 logger.error(f"Language {supplied_lang} is not supported by TED")
 
     def to_ted_langcodes(self, languages):
-        """ Converts languages queries into TED language codes
+        """Converts languages queries into TED language codes
 
-            Examples:
-                ["English", "fr", "hin"] => ["en", "fr", "hi"]
-                ["chi", "fake"] => ["zh", "zh-cn", "zh-tw"]
+        Examples:
+            ["English", "fr", "hin"] => ["en", "fr", "hi"]
+            ["chi", "fake"] => ["zh", "zh-cn", "zh-tw"]
         """
 
         lang_code_list = []
@@ -199,9 +220,9 @@ class Ted2Zim:
         return list(set(lang_code_list))
 
     def extract_videos_from_playlist(self, playlist):
-        """ extracts metadata for all videos in the given playlist
+        """extracts metadata for all videos in the given playlist
 
-            calls extract_video_info on all links to get this data
+        calls extract_video_info on all links to get this data
         """
 
         playlist_url = f"{self.playlists_base_url}/{playlist}"
@@ -310,7 +331,7 @@ class Ted2Zim:
         return lang_name
 
     def get_subtitle_dict(self, lang):
-        """ dict of language name and code from a larger dict lang
+        """dict of language name and code from a larger dict lang
 
         Example:
         {
@@ -416,6 +437,31 @@ class Ted2Zim:
             return current_lang, query
         return current_lang
 
+    def extract_download_link(self, talk_info):
+        """ Returns download link / youtube video ID for a TED video """
+
+        download_links = talk_info["downloads"]["nativeDownloads"]
+        if download_links:
+            for size in ("medium", "low", "high"):
+                if size in download_links:
+                    return download_links[size]
+            logger.error("No link to download video while we should have")
+            return None
+
+        talk_data = talk_info["player_talks"][0]
+        if talk_data.get("external", {}).get("service") == "YouTube" and talk_data[
+            "external"
+        ].get("code"):
+            logger.debug("Using Youtube link")
+            return talk_data["external"]["code"]
+
+        if talk_data.get("resources", {}).get("h264", [{"file": None}])[0]:
+            logger.debug("Using h264 resource link")
+            return talk_data["resources"]["h264"][0]["file"]
+
+        logger.error("No download link found for the video")
+        return None
+
     def update_videos_list(
         self,
         video_id,
@@ -526,13 +572,9 @@ class Ted2Zim:
         date = dateutil.parser.parse(talk_info["recorded_at"]).strftime("%d %B %Y")
         length = int(talk_info["duration"]) // 60
         thumbnail = talk_info["player_talks"][0]["thumb"]
-        download_links = talk_info["downloads"]["nativeDownloads"]
-        if not download_links:
-            logger.error("No direct download links found for the video")
-            return False
-        video_link = download_links["medium"]
+        video_link = self.extract_download_link(talk_info)
         if not video_link:
-            logger.error("No link to download video in medium quality")
+            logger.error("No suitable download link found. Skipping video")
             return False
 
         langs = talk_info["player_talks"][0]["languages"]
@@ -614,7 +656,7 @@ class Ted2Zim:
                     video["description"] = [
                         {"lang": "default", "text": video["description"][index]["text"]}
                     ] + video["description"]
-                    break
+
             if not en_found:
                 video["title"] = [
                     {"lang": "default", "text": video["title"][0]["text"]}
@@ -623,6 +665,9 @@ class Ted2Zim:
                     {"lang": "default", "text": video["description"][0]["text"]}
                 ] + video["description"]
 
+            # update video slug
+            video["slug"] = slugify(video["title"][0]["text"], separator="-")
+
     def render_video_pages(self):
 
         # Render static html pages from the scraped video data and
@@ -631,8 +676,6 @@ class Ted2Zim:
             loader=jinja2.FileSystemLoader(str(self.templates_dir)), autoescape=True
         )
         for video in self.videos:
-            video_id = str(video["id"])
-
             html = env.get_template("article.html").render(
                 speaker=video["speaker"],
                 languages=video["subtitles"],
@@ -642,11 +685,12 @@ class Ted2Zim:
                 profession=video["speaker_profession"],
                 video_format=self.video_format,
                 autoplay=self.autoplay,
-                video_id=video_id,
+                video_id=str(video["id"]),
                 titles=video["title"],
                 descriptions=video["description"],
+                back_to_list=_("Back to the list"),
             )
-            html_path = self.build_dir.joinpath(f"{video_id}.html")
+            html_path = self.build_dir.joinpath(video["slug"])
             with open(html_path, "w", encoding="utf-8") as html_page:
                 html_page.write(html)
 
@@ -666,8 +710,14 @@ class Ted2Zim:
             for key, value in all_langs.items()
         ]
         languages = sorted(languages, key=lambda x: x["languageName"])
-        html = env.get_template("home.html").render(languages=languages)
-        home_page_path = self.build_dir.joinpath("index.html")
+        html = env.get_template("home.html").render(
+            languages=languages,
+            page_title=_("TED Talks"),
+            language_filter_text=_("Filter by language"),
+            back_to_top=_("Back to the top"),
+            pagination_text=_("Page"),
+        )
+        home_page_path = self.build_dir.joinpath("index")
         with open(home_page_path, "w", encoding="utf-8") as html_page:
             html_page.write(html)
 
@@ -698,6 +748,7 @@ class Ted2Zim:
                 "description": video["description"],
                 "title": video["title"],
                 "speaker": video["speaker"],
+                "slug": video["slug"],
             }
             video_list.append(json_data)
         assets_path = self.build_dir.joinpath("assets")
@@ -707,108 +758,212 @@ class Ted2Zim:
         with open(assets_path.joinpath("data.js"), "w") as data_file:
             data_file.write("json_data = " + json.dumps(video_list, indent=4))
 
-    def download_video_files(self):
+    def download_jpeg_image_and_convert(
+        self, url, fpath, preset_options={}, resize=None
+    ):
+        """ downloads a JPEG image and converts and optimizes it into desired format detected from fpath """
+
+        org_jpeg_path = pathlib.Path(
+            tempfile.NamedTemporaryFile(delete=False, suffix=".jpg").name
+        )
+        save_large_file(url, org_jpeg_path)
+        if resize is not None:
+            resize_image(
+                org_jpeg_path,
+                width=resize[0],
+                height=resize[1],
+                method="cover",
+            )
+        optimize_image(
+            org_jpeg_path, fpath, convert=True, delete_src=True, **preset_options
+        )
+        logger.debug(f"Converted {org_jpeg_path} to {fpath} and optimized ")
+
+    def download_speaker_image(
+        self, video_id, video_title, video_speaker, speaker_path
+    ):
+        """ downloads the speaker image """
+
+        downloaded_from_cache = False
+        preset = WebpMedium()
+        if self.s3_storage:
+            s3_key = f"speaker_image/{video_id}"
+            downloaded_from_cache = self.download_from_cache(
+                s3_key, speaker_path, preset.VERSION
+            )
+        if not downloaded_from_cache:
+            try:
+                # download an image of the speaker
+                if not video_speaker:
+                    logger.debug("Speaker doesn't have an image")
+                else:
+                    logger.debug(f"Downloading Speaker image for {video_title}")
+                    self.download_jpeg_image_and_convert(
+                        video_speaker, speaker_path, preset_options=preset.options
+                    )
+            except Exception:
+                logger.error(f"Could not download speaker image for {video_title}")
+            else:
+                if self.s3_storage and video_speaker:
+                    self.upload_to_cache(s3_key, speaker_path, preset.VERSION)
+
+    def download_thumbnail(
+        self, video_id, video_title, video_thumbnail, thumbnail_path
+    ):
+        """ download the thumbnail """
+
+        downloaded_from_cache = False
+        preset = WebpMedium()
+        if self.s3_storage:
+            s3_key = f"thumbnail/{video_id}"
+            downloaded_from_cache = self.download_from_cache(
+                s3_key, thumbnail_path, preset.VERSION
+            )
+        if not downloaded_from_cache:
+            try:
+                # download the thumbnail of the video
+                logger.debug(f"Downloading thumbnail for {video_title}")
+                self.download_jpeg_image_and_convert(
+                    video_thumbnail,
+                    thumbnail_path,
+                    preset_options=preset.options,
+                    resize=(248, 187),
+                )
+            except Exception:
+                logger.error(f"Could not download thumbnail for {video_title}")
+            else:
+                if self.s3_storage:
+                    self.upload_to_cache(s3_key, thumbnail_path, preset.VERSION)
+
+    def download_video_files(self, video):
         """ download all video files (video, thumbnail, speaker) """
 
         # Download all the TED talk videos and the meta-data for it.
         # Save the videos in build_dir/{video id}/video.mp4.
         # Save the thumbnail for the video in build_dir/{video id}/thumbnail.jpg.
         # Save the image of the speaker in build_dir/{video id}/speaker.jpg.
-        for video in self.videos:
-            # set up variables
-            video_id = str(video["id"])
-            # Take the english version of title or else whatever language it's available in
-            video_title = video["title"][0]["text"]
-            video_link = video["video_link"]
-            video_speaker = video["speaker_picture"]
-            video_thumbnail = video["thumbnail"]
-            video_dir = self.videos_dir.joinpath(video_id)
-            org_video_file_path = video_dir.joinpath("video.mp4")
-            req_video_file_path = video_dir.joinpath(f"video.{self.video_format}")
-            speaker_path = video_dir.joinpath("speaker.jpg")
-            thumbnail_path = video_dir.joinpath("thumbnail.jpg")
 
-            # ensure that video directory exists
-            if not video_dir.exists():
-                video_dir.mkdir(parents=True)
+        # set up variables
+        video_id = str(video["id"])
+        # Take the english version of title or else whatever language it's available in
+        video_title = video["title"][0]["text"]
+        video_link = video["video_link"]
+        video_speaker = video["speaker_picture"]
+        video_thumbnail = video["thumbnail"]
+        video_dir = self.videos_dir.joinpath(video_id)
+        org_video_file_path = video_dir.joinpath("video.mp4")
+        req_video_file_path = video_dir.joinpath(f"video.{self.video_format}")
+        speaker_path = video_dir.joinpath("speaker.webp")
+        thumbnail_path = video_dir.joinpath("thumbnail.webp")
 
-            # set preset
-            preset = {"mp4": VideoMp4Low}.get(self.video_format, VideoWebmLow)()
+        # ensure that video directory exists
+        if not video_dir.exists():
+            video_dir.mkdir(parents=True)
 
-            # download video
-            downloaded_from_cache = False
-            logger.debug(f"Downloading {video_title}")
-            if self.s3_storage:
-                s3_key = f"{self.video_format}/{self.video_quality}/{video_id}"
-                downloaded_from_cache = self.download_from_cache(
-                    s3_key, req_video_file_path, preset.VERSION
-                )
-            if not downloaded_from_cache:
-                try:
-                    save_large_file(video_link, org_video_file_path)
-                except Exception:
-                    logger.error(f"Could not download {org_video_file_path}")
+        # set preset
+        preset = {"mp4": VideoMp4Low}.get(self.video_format, VideoWebmLow)()
 
-            # download an image of the speaker
-            if not video_speaker:
-                logger.debug("Speaker doesn't have an image")
-            else:
-                logger.debug(f"Downloading Speaker image for {video_title}")
-                save_large_file(video_speaker, speaker_path)
-
-            # download the thumbnail of the video
-            logger.debug(f"Downloading thumbnail for {video_title}")
-            save_large_file(video_thumbnail, thumbnail_path)
-
-            # recompress if necessary
+        # download video
+        downloaded_from_cache = False
+        logger.debug(f"Downloading {video_title}")
+        if self.s3_storage:
+            s3_key = f"{self.video_format}/{self.video_quality}/{video_id}"
+            downloaded_from_cache = self.download_from_cache(
+                s3_key, req_video_file_path, preset.VERSION
+            )
+        if not downloaded_from_cache:
             try:
+                if "https://" not in video_link:
+                    options = (
+                        BestWebm if self.video_format == "webm" else BestMp4
+                    ).get_options(
+                        target_dir=video_dir, filepath=pathlib.Path("video.%(ext)s")
+                    )
+                    self.yt_downloader.download(video_link, options)
+                else:
+                    save_large_file(video_link, org_video_file_path)
+            except Exception:
+                logger.error(f"Could not download {org_video_file_path}")
+
+        # download speaker and thumbnail images
+        self.download_speaker_image(video_id, video_title, video_speaker, speaker_path)
+        self.download_thumbnail(video_id, video_title, video_thumbnail, thumbnail_path)
+
+        # recompress if necessary
+        try:
+            if not downloaded_from_cache:
                 post_process_video(
                     video_dir,
                     video_id,
                     preset,
                     self.video_format,
                     self.low_quality,
-                    downloaded_from_cache,
                 )
-            except Exception as e:
-                logger.error(f"Failed to post process video {video_id}")
-                logger.debug(e)
-            else:
-                # upload to cache only if recompress was successful
-                if self.s3_storage and not downloaded_from_cache:
-                    self.upload_to_cache(s3_key, req_video_file_path, preset.VERSION)
+        except Exception as e:
+            logger.error(f"Failed to post process video {video_id}")
+            logger.debug(e)
+        else:
+            # upload to cache only if recompress was successful
+            if self.s3_storage and not downloaded_from_cache:
+                self.upload_to_cache(s3_key, req_video_file_path, preset.VERSION)
+
+    def download_video_files_parallel(self):
+        """ download videos and images parallely """
+
+        self.yt_downloader = YoutubeDownloader(threads=1)
+        with concurrent.futures.ThreadPoolExecutor(
+            max_workers=self.threads
+        ) as executor:
+            fs = [
+                executor.submit(self.download_video_files, video)
+                for video in self.videos
+            ]
+            concurrent.futures.wait(fs, return_when=concurrent.futures.ALL_COMPLETED)
+        self.yt_downloader.shutdown()
+
+    def download_subtitles(self, index, video):
+        """ download, converts and writes VTT subtitles for a video at a specific index in self.videos """
 
-    def download_subtitles(self):
-        """ download, converts and writes VTT subtitles for all videos """
         # Download the subtitle files, generate a WebVTT file
         # and save the subtitles in
         # build_dir/{video id}/subs/subs_{language code}.vtt
-        for index, video in enumerate(self.videos):
-            if not video["subtitles"]:
-                continue
-            video_dir = self.videos_dir.joinpath(video["id"])
-            subs_dir = video_dir.joinpath("subs")
-            if not subs_dir.exists():
-                subs_dir.mkdir(parents=True)
-            else:
-                logger.debug(f"Subs dir exists already")
+        if not video["subtitles"]:
+            return
+        video_dir = self.videos_dir.joinpath(video["id"])
+        subs_dir = video_dir.joinpath("subs")
+        if not subs_dir.exists():
+            subs_dir.mkdir(parents=True)
+        else:
+            logger.debug(f"Subs dir exists already")
 
-            # download subtitles
-            logger.debug(f"Downloading subtitles for {video['title'][0]['text']}")
-            valid_subs = []
-            for subtitle in video["subtitles"]:
-                time.sleep(0.5)  # throttling
-                vtt_subtitle = WebVTT(subtitle["link"]).convert()
-                if not vtt_subtitle:
-                    logger.error(
-                        f"Subtitle file for {subtitle['languageCode']} could not be created"
-                    )
-                    continue
-                valid_subs.append(subtitle)
-                vtt_path = subs_dir.joinpath(f"subs_{subtitle['languageCode']}.vtt")
-                with open(vtt_path, "w", encoding="utf-8") as sub_file:
-                    sub_file.write(vtt_subtitle)
-            self.videos[index]["subtitles"] = valid_subs
+        # download subtitles
+        logger.debug(f"Downloading subtitles for {video['title'][0]['text']}")
+        valid_subs = []
+        for subtitle in video["subtitles"]:
+            time.sleep(0.5)  # throttling
+            vtt_subtitle = WebVTT(subtitle["link"]).convert()
+            if not vtt_subtitle:
+                logger.error(
+                    f"Subtitle file for {subtitle['languageCode']} could not be created"
+                )
+                continue
+            valid_subs.append(subtitle)
+            vtt_path = subs_dir.joinpath(f"subs_{subtitle['languageCode']}.vtt")
+            with open(vtt_path, "w", encoding="utf-8") as sub_file:
+                sub_file.write(vtt_subtitle)
+        self.videos[index]["subtitles"] = valid_subs
+
+    def download_subtitles_parallel(self):
+        """ download subtitles for all videos parallely """
+
+        with concurrent.futures.ThreadPoolExecutor(
+            max_workers=self.threads
+        ) as executor:
+            fs = [
+                executor.submit(self.download_subtitles, index, video)
+                for index, video in enumerate(self.videos)
+            ]
+            concurrent.futures.wait(fs, return_when=concurrent.futures.ALL_COMPLETED)
 
     def s3_credentials_ok(self):
         logger.info("Testing S3 Optimization Cache credentials")
@@ -824,7 +979,7 @@ class Ted2Zim:
             return False
         return True
 
-    def download_from_cache(self, key, video_path, encoder_version):
+    def download_from_cache(self, key, object_path, encoder_version):
         """ whether it downloaded from S3 cache """
 
         if self.use_any_optimized_version:
@@ -835,26 +990,26 @@ class Ted2Zim:
                 key, tag="encoder_version", value=f"v{encoder_version}"
             ):
                 return False
-        video_path.parent.mkdir(parents=True, exist_ok=True)
+        object_path.parent.mkdir(parents=True, exist_ok=True)
         try:
-            self.s3_storage.download_file(key, video_path)
+            self.s3_storage.download_file(key, object_path)
         except Exception as exc:
             logger.error(f"{key} failed to download from cache: {exc}")
             return False
-        logger.info(f"downloaded {video_path} from cache at {key}")
+        logger.info(f"downloaded {object_path} from cache at {key}")
         return True
 
-    def upload_to_cache(self, key, video_path, encoder_version):
+    def upload_to_cache(self, key, object_path, encoder_version):
         """ whether it uploaded from S3 cache """
 
         try:
             self.s3_storage.upload_file(
-                video_path, key, meta={"encoder_version": f"v{encoder_version}"}
+                object_path, key, meta={"encoder_version": f"v{encoder_version}"}
             )
         except Exception as exc:
             logger.error(f"{key} failed to upload to cache: {exc}")
             return False
-        logger.info(f"uploaded {video_path} to cache at {key}")
+        logger.info(f"uploaded {object_path} to cache at {key}")
         return True
 
     def remove_failed_topics_and_check_extraction(self, failed_topics):
@@ -899,8 +1054,8 @@ class Ted2Zim:
 
         self.add_default_language()
         self.update_zim_metadata()
-        self.download_video_files()
-        self.download_subtitles()
+        self.download_video_files_parallel()
+        self.download_subtitles_parallel()
         self.render_home_page()
         self.render_video_pages()
         self.copy_files_to_build_directory()
@@ -918,7 +1073,7 @@ class Ted2Zim:
                 build_dir=self.build_dir,
                 fpath=self.output_dir.joinpath(self.fname),
                 name=self.name,
-                main_page="index.html",
+                main_page="index",
                 favicon="favicon.png",
                 title=self.title,
                 description=self.description,
